@@ -30,8 +30,10 @@ extern int		_XInitImageFuncPtrs(XImage *image);
  * Forward declarations
  */
 
+#ifndef MAC_OSX_TK
 static void		BlendComplexAlpha(XImage *bgImg, PhotoInstance *iPtr,
 			    int xOffset, int yOffset, int width, int height);
+#endif
 static int		IsValidPalette(PhotoInstance *instancePtr,
 			    const char *palette);
 static int		CountBits(pixel mask);
@@ -409,7 +411,7 @@ TkImgPhotoGet(
  *
  *----------------------------------------------------------------------
  */
-
+#ifndef MAC_OSX_TK
 #ifndef _WIN32
 #define GetRValue(rgb)	(UCHAR(((rgb) & red_mask) >> red_shift))
 #define GetGValue(rgb)	(UCHAR(((rgb) & green_mask) >> green_shift))
@@ -418,13 +420,6 @@ TkImgPhotoGet(
 	(UCHAR(r) << red_shift)   | \
 	(UCHAR(g) << green_shift) | \
 	(UCHAR(b) << blue_shift)  ))
-#ifdef MAC_OSX_TK
-#define RGBA(r, g, b, a) ((unsigned)( \
-	(UCHAR(r) << red_shift)   | \
-	(UCHAR(g) << green_shift) | \
-	(UCHAR(b) << blue_shift)  | \
-	(UCHAR(a) << alpha_shift) ))
-#endif
 #define RGB15(r, g, b)	((unsigned)( \
 	(((r) * red_mask / 255)   & red_mask)   | \
 	(((g) * green_mask / 255) & green_mask) | \
@@ -443,16 +438,7 @@ BlendComplexAlpha(
     unsigned long pixel;
     unsigned char r, g, b, alpha, unalpha, *masterPtr;
     unsigned char *alphaAr = iPtr->masterPtr->pix32;
-#if defined(MAC_OSX_TK)
-    /* Background "pixels" are actually 2^pp x 2^pp blocks of subpixels.  Each
-     * block gets blended with the color of one image pixel.  Since we iterate
-     * over the background subpixels, we reset the width and height to the
-     * subpixel dimensions of the background image we are using.
-     */
-    int pp = bgImg->pixelpower;
-    width = width << pp;
-    height = height << pp;
-#endif
+
     /*
      * This blending is an integer version of the Source-Over compositing rule
      * (see Porter&Duff, "Compositing Digital Images", proceedings of SIGGRAPH
@@ -492,13 +478,6 @@ BlendComplexAlpha(
     while ((0x0001 & (blue_mask >> blue_shift)) == 0) {
 	blue_shift++;
     }
-#ifdef MAC_OSX_TK
-    unsigned long alpha_mask = visual->alpha_mask;
-    unsigned long alpha_shift = 0;
-    while ((0x0001 & (alpha_mask >> alpha_shift)) == 0) {
-	alpha_shift++;
-    }
-#endif
 #endif /* !_WIN32 */
 
     /*
@@ -558,16 +537,9 @@ BlendComplexAlpha(
 #endif /* !_WIN32 && !MAC_OSX_TK */
 
     for (y = 0; y < height; y++) {
-# if !defined(MAC_OSX_TK)
 	line = (y + yOffset) * iPtr->masterPtr->width;
 	for (x = 0; x < width; x++) {
 	    masterPtr = alphaAr + ((line + x + xOffset) * 4);
-#else
-	/* Repeat each image row and column 2^pp times. */
-	line = ((y>>pp) + yOffset) * iPtr->masterPtr->width;
-	for (x = 0; x < width; x++) {
-	    masterPtr = alphaAr + ((line + (x>>pp) + xOffset) * 4);
-#endif
 	    alpha = masterPtr[3];
 
 	    /*
@@ -599,16 +571,13 @@ BlendComplexAlpha(
 		    g = ALPHA_BLEND(ga, g, alpha, unalpha);
 		    b = ALPHA_BLEND(ba, b, alpha, unalpha);
 		}
-#ifndef MAC_OSX_TK
 		XPutPixel(bgImg, x, y, RGB(r, g, b));
-#else
-		XPutPixel(bgImg, x, y, RGBA(r, g, b, alpha));
-#endif
 	    }
 	}
     }
 #undef ALPHA_BLEND
 }
+#endif /* MAC_OSX_TK */
 
 /*
  *----------------------------------------------------------------------
@@ -651,6 +620,24 @@ TkImgPhotoDisplay(
 	return;
     }
 
+#ifdef MAC_OSX_TK
+    /*
+     * The Mac version of TkPutImage handles RGBA images directly.  There is
+     * no need to call XGetImage or to do the Porter-Duff compositing by hand.
+     * We just let the CG graphics library do it, using the graphics hardware.
+     */
+    unsigned char *rgbaPixels = instancePtr->masterPtr->pix32;
+
+    XImage *photo = XCreateImage(display, NULL, 32, ZPixmap, 0, (char*)rgbaPixels,
+				 (unsigned int)instancePtr->width,
+				 (unsigned int)instancePtr->height,
+				 0, (unsigned int)(4 * instancePtr->width));
+    TkPutImage(NULL, 0, display, drawable, instancePtr->gc,
+	       photo, imageX, imageY, drawableX, drawableY,
+	       (unsigned int) width, (unsigned int) height);
+    photo->data = NULL;
+    XDestroyImage(photo);
+#else
     if ((instancePtr->masterPtr->flags & COMPLEX_ALPHA)
 	    && visInfo.depth >= 15
 	    && (visInfo.class == DirectColor || visInfo.class == TrueColor)) {
@@ -709,6 +696,7 @@ TkImgPhotoDisplay(
 	XSetClipOrigin(display, instancePtr->gc, 0, 0);
     }
     XFlush(display);
+#endif
 }
 
 /*
