@@ -81,8 +81,9 @@ static Tk_ImageType imageType = {
  * Forward declarations for functions defined later in this file:
  */
 
-static int              ImageCmd(ClientData dummy,
-                            Tcl_Interp *interp, int argc, const char **argv);
+static int              ImageObjCmd(ClientData dummy,
+                            Tcl_Interp *interp, int objc,
+            			    Tcl_Obj * const objv[]);
 
 
 /*
@@ -171,11 +172,11 @@ ImageCreate(
     timPtr->interp = interp;
     timPtr->width = 30;
     timPtr->height = 15;
-    timPtr->imageName = ckalloc((unsigned) (strlen(name) + 1));
+    timPtr->imageName = ckalloc(strlen(name) + 1);
     strcpy(timPtr->imageName, name);
-    timPtr->varName = ckalloc((unsigned) (strlen(varName) + 1));
+    timPtr->varName = ckalloc(strlen(varName) + 1);
     strcpy(timPtr->varName, varName);
-    Tcl_CreateCommand(interp, name, ImageCmd, timPtr, NULL);
+    Tcl_CreateObjCommand(interp, name, ImageObjCmd, timPtr, NULL);
     *clientDataPtr = timPtr;
     Tk_ImageChanged(master, 0, 0, 30, 15, 30, 15);
     return TCL_OK;
@@ -184,7 +185,7 @@ ImageCreate(
 /*
  *----------------------------------------------------------------------
  *
- * ImageCmd --
+ * ImageObjCmd --
  *
  *	This function implements the commands corresponding to individual
  *	images.
@@ -200,38 +201,37 @@ ImageCreate(
 
 	/* ARGSUSED */
 static int
-ImageCmd(
+ImageObjCmd(
     ClientData clientData,	/* Main window for application. */
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const objv[])		/* Argument strings. */
 {
     TImageMaster *timPtr = clientData;
     int x, y, width, height;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], "option ?arg ...?", NULL);
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
 	return TCL_ERROR;
     }
-    if (strcmp(argv[1], "changed") == 0) {
-	if (argc != 8) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		    " changed x y width height imageWidth imageHeight", NULL);
+    if (strcmp(Tcl_GetString(objv[1]), "changed") == 0) {
+	if (objc != 8) {
+	    Tcl_WrongNumArgs(interp, 1, objv, "changed x y width height"
+		    " imageWidth imageHeight");
 	    return TCL_ERROR;
 	}
-	if ((Tcl_GetInt(interp, argv[2], &x) != TCL_OK)
-		|| (Tcl_GetInt(interp, argv[3], &y) != TCL_OK)
-		|| (Tcl_GetInt(interp, argv[4], &width) != TCL_OK)
-		|| (Tcl_GetInt(interp, argv[5], &height) != TCL_OK)
-		|| (Tcl_GetInt(interp, argv[6], &timPtr->width) != TCL_OK)
-		|| (Tcl_GetInt(interp, argv[7], &timPtr->height) != TCL_OK)) {
+	if ((Tcl_GetIntFromObj(interp, objv[2], &x) != TCL_OK)
+		|| (Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK)
+		|| (Tcl_GetIntFromObj(interp, objv[4], &width) != TCL_OK)
+		|| (Tcl_GetIntFromObj(interp, objv[5], &height) != TCL_OK)
+		|| (Tcl_GetIntFromObj(interp, objv[6], &timPtr->width) != TCL_OK)
+		|| (Tcl_GetIntFromObj(interp, objv[7], &timPtr->height) != TCL_OK)) {
 	    return TCL_ERROR;
 	}
 	Tk_ImageChanged(timPtr->master, x, y, width, height, timPtr->width,
 		timPtr->height);
     } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1],
+	Tcl_AppendResult(interp, "bad option \"", Tcl_GetString(objv[1]),
 		"\": must be changed", NULL);
 	return TCL_ERROR;
     }
