@@ -1192,6 +1192,19 @@ ScaleVarProc(
     Tcl_Obj *valuePtr;
     int result;
 
+    /*
+     * See ticket [5d991b82].
+     */
+
+    if (scalePtr->varNamePtr == NULL) {
+	if (!(flags & TCL_INTERP_DESTROYED)) {
+	    Tcl_UntraceVar2(interp, name1, name2,
+		    TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
+		    ScaleVarProc, clientData);
+	}
+	return NULL;
+    }
+
     /*
      * If the variable is unset, then immediately recreate it unless the whole
      * interpreter is going away.
