@@ -12,6 +12,7 @@
  */
 
 #include "tkMacOSXPrivate.h"
+#include "tkMacOSXConstants.h"
 #include "tkSelect.h"
 
 static NSInteger changeCount = -1;
@@ -20,8 +21,10 @@ static Tk_Window clipboardOwner = NULL;
 #pragma mark TKApplication(TKClipboard)
 
 @implementation TKApplication(TKClipboard)
-- (void)tkProvidePasteboard:(TkDisplay *)dispPtr
-	pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type {
+- (void) tkProvidePasteboard: (TkDisplay *) dispPtr
+	pasteboard: (NSPasteboard *) sender
+	provideDataForType: (NSString *) type
+{
     NSMutableString *string = [NSMutableString new];
 
     if (dispPtr && dispPtr->clipboardActive &&
@@ -35,6 +38,7 @@ static Tk_Window clipboardOwner = NULL;
 		    NSString *s = [[NSString alloc] initWithBytesNoCopy:
 			    cbPtr->buffer length:cbPtr->length
 			    encoding:NSUTF8StringEncoding freeWhenDone:NO];
+
 		    [string appendString:s];
 		    [s release];
 		}
@@ -45,25 +49,30 @@ static Tk_Window clipboardOwner = NULL;
     [sender setString:string forType:type];
     [string release];
 }
-- (void)tkProvidePasteboard:(TkDisplay *)dispPtr {
+
+- (void) tkProvidePasteboard: (TkDisplay *) dispPtr
+{
     if (dispPtr && dispPtr->clipboardActive) {
 	[self tkProvidePasteboard:dispPtr
 		pasteboard:[NSPasteboard generalPasteboard]
 		provideDataForType:NSStringPboardType];
     }
 }
-- (void)pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type {
+
+- (void) pasteboard: (NSPasteboard *) sender
+	provideDataForType: (NSString *) type
+{
     [self tkProvidePasteboard:TkGetDisplayList() pasteboard:sender
 	    provideDataForType:type];
 }
-- (void)tkCheckPasteboard {
+
+- (void) tkCheckPasteboard
+{
     if (clipboardOwner && [[NSPasteboard generalPasteboard] changeCount] !=
 	    changeCount) {
 	TkDisplay *dispPtr = TkGetDisplayList();
-
 	if (dispPtr) {
 	    XEvent event;
-
 	    event.xany.type = SelectionClear;
 	    event.xany.serial = NextRequest(Tk_Display(clipboardOwner));
 	    event.xany.send_event = False;
@@ -115,8 +124,10 @@ TkSelGetSelection(
     int result = TCL_ERROR;
     TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
-    if (dispPtr && selection == dispPtr->clipboardAtom && (target == XA_STRING
-	    || target == dispPtr->utf8Atom)) {
+    int haveExternalClip = ([[NSPasteboard generalPasteboard] changeCount] != changeCount);
+    if (dispPtr && (haveExternalClip || dispPtr->clipboardActive)
+	        && selection == dispPtr->clipboardAtom
+	        && (target == XA_STRING || target == dispPtr->utf8Atom)) {
 	NSString *string = nil;
 	NSPasteboard *pb = [NSPasteboard generalPasteboard];
 	NSString *type = [pb availableTypeFromArray:[NSArray arrayWithObject:
@@ -125,12 +136,13 @@ TkSelGetSelection(
 	if (type) {
 	    string = [pb stringForType:type];
 	}
-	result = proc(clientData, interp, string ? (char*)[string UTF8String]
-		: "");
+	result = proc(clientData, interp, string ? [string UTF8String] : "");
     } else {
-	Tcl_AppendResult(interp, Tk_GetAtomName(tkwin, selection),
-		" selection doesn't exist or form \"",
-		Tk_GetAtomName(tkwin, target), "\" not defined", NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"%s selection doesn't exist or form \"%s\" not defined",
+		Tk_GetAtomName(tkwin, selection),
+		Tk_GetAtomName(tkwin, target)));
+	Tcl_SetErrorCode(interp, "TK", "SELECTION", "EXISTS", NULL);
     }
     return result;
 }
@@ -221,6 +233,7 @@ TkSelUpdateClipboard(
 				/* Info about the content. */
 {
     NSPasteboard *pb = [NSPasteboard generalPasteboard];
+
     changeCount = [pb addTypes:[NSArray arrayWithObject:NSStringPboardType]
 	    owner:NSApp];
 }
@@ -276,28 +289,6 @@ TkSelPropProc(
 {
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TkSuspendClipboard --
- *
- *	Handle clipboard conversion as required by the suppend event.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	The local scrap is moved to the global scrap.
- *
- *----------------------------------------------------------------------
- */
-
-void
-TkSuspendClipboard(void)
-{
-    changeCount = [[NSPasteboard generalPasteboard] changeCount];
-}
-
 /*
  * Local Variables:
  * mode: objc
