@@ -68,11 +68,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
     TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, notification);
 #endif
     BOOL movedOnly = [[notification name]
-	    isEqualToString:NSWindowDidMoveNotification];
-
-    if (movedOnly) {
-	/* constraining to screen after move not needed with AppKit */
-    }
+			 isEqualToString:NSWindowDidMoveNotification];
 
     NSWindow *w = [notification object];
     TkWindow *winPtr = TkMacOSXGetTkWindow(w);
@@ -96,6 +92,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
 	    flags |= TK_SIZE_CHANGED;
 	}
 	if (Tcl_GetServiceMode() != TCL_SERVICE_NONE) {
+
 	    /*
 	     * Propagate geometry changes immediately.
 	     */
@@ -119,6 +116,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
 		TkMacOSXIsWindowZoomed(winPtr) ? ZoomState : NormalState;
 	Tk_MapWindow((Tk_Window) winPtr);
 	if (Tcl_GetServiceMode() != TCL_SERVICE_NONE) {
+
 	    /*
 	     * Process all Tk events generated by Tk_MapWindow().
 	     */
@@ -174,6 +172,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
     return (winPtr ? NO : YES);
 }
 
+
 #ifdef TK_MAC_DEBUG_NOTIFICATIONS
 
 - (void) windowDragStart: (NSNotification *) notification
@@ -213,6 +212,8 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
 	//Tk_UnmapWindow((Tk_Window) winPtr);
     }
 }
+
+
 #endif /* TK_MAC_DEBUG_NOTIFICATIONS */
 
 - (void) _setupWindowNotifications
@@ -236,6 +237,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
     observe(NSWindowDidOrderOffScreenNotification, windowUnmapped:);
 #endif
 #undef observe
+
 }
 @end
 
@@ -256,7 +258,6 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
 #ifdef TK_MAC_DEBUG_NOTIFICATIONS
     TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, notification);
 #endif
-    TkSuspendClipboard();
 }
 
 - (void) applicationShowHide: (NSNotification *) notification
@@ -292,6 +293,31 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
 @end
 
 #pragma mark -
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpAppIsDrawing --
+ *
+ *      A widget display procedure can call this to determine whether it
+ *      is being run inside of the drawRect method.  This is needed for
+ *      some tests, especially of the Text widget, which record data in
+ *      a global Tcl variable and assume that display procedures will be
+ *      run in a predictable sequence as Tcl idle tasks.
+ *
+ * Results:
+ *	True only while running the drawRect method of a TKContentView;
+ *
+ * Side effects:
+ *	None
+ *
+ *----------------------------------------------------------------------
+ */
+MODULE_SCOPE Bool
+TkpAppIsDrawing(void) {
+    return [NSApp isDrawing];
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -299,7 +325,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
  * GenerateUpdates --
  *
  *	Given a Macintosh update region and a Tk window this function geneates
- *	a X Expose event for the window if it is within the update region. The
+ *	an X Expose event for the window if it meets the update region. The
  *	function will then recursivly have each damaged window generate Expose
  *	events for its child windows.
  *
@@ -360,7 +386,7 @@ GenerateUpdates(
     Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
 
 #ifdef TK_MAC_DEBUG_DRAWING
-    NSLog(@"Expose %p {{%d, %d}, {%d, %d}}", event.xany.window, event.xexpose.x,
+    TKLog(@"Expose %p {{%d, %d}, {%d, %d}}", event.xany.window, event.xexpose.x,
 	event.xexpose.y, event.xexpose.width, event.xexpose.height);
 #endif
 
@@ -579,10 +605,12 @@ TkGenWMConfigureEvent(
 	if ((flags & TK_SIZE_CHANGED) && !(wmPtr->flags & WM_SYNC_PENDING) &&
 		((width != Tk_Width(tkwin)) || (height != Tk_Height(tkwin)))) {
 	    if ((wmPtr->width == -1) && (width == winPtr->reqWidth)) {
+
 		/*
 		 * Don't set external width, since the user didn't change it
 		 * from what the widgets asked for.
 		 */
+
 	    } else if (wmPtr->gridWin != NULL) {
 		wmPtr->width = wmPtr->reqGridWidth
 			+ (width - winPtr->reqWidth)/wmPtr->widthInc;
@@ -594,10 +622,12 @@ TkGenWMConfigureEvent(
 	    }
 
 	    if ((wmPtr->height == -1) && (height == winPtr->reqHeight)) {
+
 		/*
 		 * Don't set external height, since the user didn't change it
 		 * from what the widgets asked for.
 		 */
+
 	    } else if (wmPtr->gridWin != NULL) {
 		wmPtr->height = wmPtr->reqGridHeight
 			+ (height - winPtr->reqHeight)/wmPtr->heightInc;
@@ -796,131 +826,216 @@ ConfigureRestrictProc(
     const NSRect *rectsBeingDrawn;
     NSInteger rectsBeingDrawnCount;
 
-    [self getRectsBeingDrawn:&rectsBeingDrawn count:&rectsBeingDrawnCount];
-
 #ifdef TK_MAC_DEBUG_DRAWING
-    TKLog(@"-[%@(%p) %s%@]", [self class], self, _cmd, NSStringFromRect(rect));
-    [[NSColor colorWithDeviceRed:0.0 green:1.0 blue:0.0 alpha:.1] setFill];
-    NSRectFillListUsingOperation(rectsBeingDrawn, rectsBeingDrawnCount,
-	    NSCompositeSourceOver);
+    TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
+    if (winPtr) fprintf(stderr, "drawRect: drawing %s\n",
+			Tk_PathName(winPtr));
 #endif
 
+    if ([NSApp simulateDrawing]) {
+    	return;
+    }
+
+    /*
+     * We do not allow recursive calls to drawRect, but we only log
+     * them on OSX > 10.13, where they should never happen.
+     */
+
+    if ([NSApp isDrawing] && [NSApp macMinorVersion] > 13) {
+	TKLog(@"WARNING: a recursive call to drawRect was aborted.");
+	return;
+    }
+
+    [NSApp setIsDrawing: YES];
+
+    [self getRectsBeingDrawn:&rectsBeingDrawn count:&rectsBeingDrawnCount];
     CGFloat height = [self bounds].size.height;
     HIMutableShapeRef drawShape = HIShapeCreateMutable();
 
     while (rectsBeingDrawnCount--) {
 	CGRect r = NSRectToCGRect(*rectsBeingDrawn++);
+
+#ifdef TK_MAC_DEBUG_DRAWING
+	fprintf(stderr, "drawRect: %dx%d@(%d,%d)\n", (int)r.size.width,
+	       (int)r.size.height, (int)r.origin.x, (int)r.origin.y);
+#endif
+
 	r.origin.y = height - (r.origin.y + r.size.height);
 	HIShapeUnionWithRect(drawShape, &r);
     }
-    if (CFRunLoopGetMain() == CFRunLoopGetCurrent()) {
-	[self generateExposeEvents:(HIShapeRef)drawShape];
-    } else {
-	[self performSelectorOnMainThread:@selector(generateExposeEvents:)
-		withObject:(id)drawShape waitUntilDone:NO
-		modes:[NSArray arrayWithObjects:NSRunLoopCommonModes,
-
-			NSEventTrackingRunLoopMode, NSModalPanelRunLoopMode,
-			nil]];
-    }
-
+    [self generateExposeEvents:(HIShapeRef)drawShape];
     CFRelease(drawShape);
+    [NSApp setIsDrawing: NO];
+
+#ifdef TK_MAC_DEBUG_DRAWING
+    fprintf(stderr, "drawRect: done.\n");
+#endif
 }
 
 -(void) setFrameSize: (NSSize)newsize
 {
     [super setFrameSize: newsize];
-    if ([self inLiveResize]) {
-	NSWindow *w = [self window];
-	TkWindow *winPtr = TkMacOSXGetTkWindow(w);
-	Tk_Window tkwin = (Tk_Window) winPtr;
+    NSWindow *w = [self window];
+    TkWindow *winPtr = TkMacOSXGetTkWindow(w);
+    Tk_Window tkwin = (Tk_Window) winPtr;
+    if (winPtr) {
+	/* On OSX versions below 10.14 setFrame calls drawRect.
+	 * On 10.14 it does its own drawing.
+	 */
+	if ([NSApp macMinorVersion] > 13) {
+	    [NSApp setIsDrawing:YES];
+	}
 	unsigned int width = (unsigned int)newsize.width;
 	unsigned int height=(unsigned int)newsize.height;
 	ClientData oldArg;
     	Tk_RestrictProc *oldProc;
 
-	/* This can be called from outside the Tk event loop.
+	/*
+	 * This can be called from outside the Tk event loop.
 	 * Since it calls Tcl_DoOneEvent, we need to make sure we
 	 * don't clobber the AutoreleasePool set up by the caller.
 	 */
+
 	[NSApp _lockAutoreleasePool];
 
 	/*
-	 * Try to prevent flickers and flashes.
+	 * Disable Tk drawing until the window has been completely configured.
 	 */
-	[w disableFlushWindow];
-	NSDisableScreenUpdates();
 
-	/* Disable Tk drawing until the window has been completely configured.*/
 	TkMacOSXSetDrawingEnabled(winPtr, 0);
 
-	 /* Generate and handle a ConfigureNotify event for the new size.*/
+	 /*
+	  * Generate and handle a ConfigureNotify event for the new size.
+	  */
+
 	TkGenWMConfigureEvent(tkwin, Tk_X(tkwin), Tk_Y(tkwin), width, height,
 			      TK_SIZE_CHANGED | TK_MACOSX_HANDLE_EVENT_IMMEDIATELY);
     	oldProc = Tk_RestrictEvents(ConfigureRestrictProc, NULL, &oldArg);
-	while (Tk_DoOneEvent(TK_X_EVENTS|TK_DONT_WAIT)) {}
     	Tk_RestrictEvents(oldProc, oldArg, &oldArg);
 
-	/* Now that Tk has configured all subwindows we can create the clip regions. */
+	/*
+	 * Now that Tk has configured all subwindows, create the clip regions.
+	 */
+
 	TkMacOSXSetDrawingEnabled(winPtr, 1);
 	TkMacOSXInvalClipRgns(tkwin);
 	TkMacOSXUpdateClipRgn(winPtr);
 
-	 /* Finally, generate and process expose events to redraw the window. */
+	 /*
+	  * Generate and process expose events to redraw the window.
+	  */
+
 	HIRect bounds = NSRectToCGRect([self bounds]);
 	HIShapeRef shape = HIShapeCreateWithRect(&bounds);
 	[self generateExposeEvents: shape];
-	while (Tk_DoOneEvent(TK_ALL_EVENTS|TK_DONT_WAIT)) {}
-	[w enableFlushWindow];
-	[w flushWindowIfNeeded];
-	NSEnableScreenUpdates();
+	[w displayIfNeeded];
+
+	/*
+	 * Finally, unlock the main autoreleasePool.
+	 */
+	
 	[NSApp _unlockAutoreleasePool];
     }
 }
 
 /*
- * As insurance against bugs that might cause layout glitches during a live
- * resize, we redraw the window one more time at the end of the resize
- * operation.
+ * Core method of this class: generates expose events for redrawing.  The
+ * expose events are immediately removed from the Tcl event loop and processed.
+ * This causes drawing procedures to be scheduled as idle events.  Then all
+ * pending idle events are processed so the drawing will actually take place.
  */
 
-- (void)viewDidEndLiveResize
-{
-    HIRect bounds = NSRectToCGRect([self bounds]);
-    HIShapeRef shape = HIShapeCreateWithRect(&bounds);
-    [super viewDidEndLiveResize];
-    [self generateExposeEvents: shape];
-}
-
-/* Core method of this class: generates expose events for redrawing.  If the
- * Tcl_ServiceMode is set to TCL_SERVICE_ALL then the expose events will be
- * immediately removed from the Tcl event loop and processed.  Typically, they
- * should be queued, however.
- */
 - (void) generateExposeEvents: (HIShapeRef) shape
 {
     unsigned long serial;
     CGRect updateBounds;
     int updatesNeeded;
     TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
-
+    ClientData oldArg;
+    Tk_RestrictProc *oldProc;
     if (!winPtr) {
-		return;
+	return;
     }
 
-    /* Generate Tk Expose events. */
+    /*
+     * Generate Tk Expose events.
+     */
+
     HIShapeGetBounds(shape, &updateBounds);
-    /* All of these events will share the same serial number. */
+
+    /*
+     * All of these events will share the same serial number.
+     */
+
     serial = LastKnownRequestProcessed(Tk_Display(winPtr));
     updatesNeeded = GenerateUpdates(shape, &updateBounds, winPtr);
 
-    /* Process the Expose events if the service mode is TCL_SERVICE_ALL */
-    if (updatesNeeded && Tcl_GetServiceMode() == TCL_SERVICE_ALL) {
-	ClientData oldArg;
-    	Tk_RestrictProc *oldProc = Tk_RestrictEvents(ExposeRestrictProc,
-						     UINT2PTR(serial), &oldArg);
-    	while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {}
+    if (updatesNeeded) {
+
+	/*
+	 * First process all of the Expose events.
+	 */
+
+    	oldProc = Tk_RestrictEvents(ExposeRestrictProc, UINT2PTR(serial), &oldArg);
+    	while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {};
     	Tk_RestrictEvents(oldProc, oldArg, &oldArg);
+
+	/*
+	 * Starting with OSX 10.14, which uses Core Animation to draw windows,
+	 * all drawing must be done within the drawRect method.  (The CGContext
+	 * which draws to the backing CALayer is created by the NSView before
+	 * calling drawRect, and destroyed when drawRect returns.  Drawing done
+	 * with the current CGContext outside of the drawRect method has no
+	 * effect.)
+	 *
+	 * Fortunately, Tk schedules all drawing to be done while Tcl is idle.
+	 * So we can do the drawing by processing all of the idle events that
+	 * were created when the expose events were processed.
+	 */
+	while (Tcl_DoOneEvent(TCL_IDLE_EVENTS)) {}
+    }
+}
+
+/*
+ * This method is called when a user changes between light and dark mode.
+ * The implementation here generates a Tk virtual event which can be bound
+ * to a function that redraws the window in an appropriate style.
+ */
+
+- (void) viewDidChangeEffectiveAppearance
+{
+    XVirtualEvent event;
+    int x, y;
+    NSString *osxMode = [[NSUserDefaults standardUserDefaults] stringForKey:@"AppleInterfaceStyle"];
+    NSWindow *w = [self window];
+    TkWindow *winPtr = TkMacOSXGetTkWindow(w);
+    Tk_Window tkwin = (Tk_Window) winPtr;
+
+    if (!winPtr) {
+	return;
+    }
+    bzero(&event, sizeof(XVirtualEvent));
+    event.type = VirtualEvent;
+    event.serial = LastKnownRequestProcessed(Tk_Display(tkwin));
+    event.send_event = false;
+    event.display = Tk_Display(tkwin);
+    event.event = Tk_WindowId(tkwin);
+    event.root = XRootWindow(Tk_Display(tkwin), 0);
+    event.subwindow = None;
+    event.time = TkpGetMS();
+    XQueryPointer(NULL, winPtr->window, NULL, NULL,
+    		  &event.x_root, &event.y_root, &x, &y, &event.state);
+    Tk_TopCoordsToWindow(tkwin, x, y, &event.x, &event.y);
+    event.same_screen = true;
+    if (osxMode == nil) {
+	event.name = Tk_GetUid("LightAqua");
+	Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
+	return;
+    }
+    if ([osxMode isEqual:@"Dark"]) {
+	event.name = Tk_GetUid("DarkAqua");
+	Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
+	return;
     }
 }
 
@@ -928,6 +1043,7 @@ ConfigureRestrictProc(
  * This is no-op on 10.7 and up because Apple has removed this widget,
  * but we are leaving it here for backwards compatibility.
  */
+
 - (void) tkToolbarButton: (id) sender
 {
 #ifdef TK_MAC_DEBUG_EVENTS
@@ -937,6 +1053,9 @@ ConfigureRestrictProc(
     int x, y;
     TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
     Tk_Window tkwin = (Tk_Window) winPtr;
+    if (!winPtr){
+	return;
+    }
     bzero(&event, sizeof(XVirtualEvent));
     event.type = VirtualEvent;
     event.serial = LastKnownRequestProcessed(Tk_Display(tkwin));
@@ -957,7 +1076,6 @@ ConfigureRestrictProc(
 - (BOOL) isOpaque
 {
     NSWindow *w = [self window];
-
       return (w && (([w styleMask] & NSTexturedBackgroundWindowMask) ||
     	    ![w isOpaque]) ? NO : YES);
 }
