@@ -3,7 +3,7 @@
  *
  *	This file contains most of the X calls called by Tk. Many of these
  *	calls are just stubs and either don't make sense on the Macintosh or
- *	their implamentation just doesn't do anything. Other calls will
+ *	their implementation just doesn't do anything. Other calls will
  *	eventually be moved into other files.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
@@ -40,10 +40,13 @@ CGFloat tkMacOSXZeroScreenTop = 0;
  * Declarations of static variables used in this file.
  */
 
+/* The unique Macintosh display. */
 static TkDisplay *gMacDisplay = NULL;
-				/* Macintosh display. */
+/* The default name of the Macintosh display. */
 static const char *macScreenName = ":0";
-				/* Default name of macintosh display. */
+/* Timestamp showing the last reset of the inactivity timer. */
+static Time lastInactivityReset = 0;
+
 
 /*
  * Forward declarations of procedures used in this file.
@@ -175,7 +178,7 @@ TkpOpenDisplay(
     {
 	int major, minor, patch;
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1080
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 101000
 	Gestalt(gestaltSystemVersionMajor, (SInt32*)&major);
 	Gestalt(gestaltSystemVersionMinor, (SInt32*)&minor);
 	Gestalt(gestaltSystemVersionBugFix, (SInt32*)&patch);
@@ -200,7 +203,6 @@ TkpOpenDisplay(
     screen->root_visual = ckalloc(sizeof(Visual));
     screen->root_visual->visualid     = 0;
     screen->root_visual->class	      = TrueColor;
-    screen->root_visual->alpha_mask   = 0xFF000000;
     screen->root_visual->red_mask     = 0x00FF0000;
     screen->root_visual->green_mask   = 0x0000FF00;
     screen->root_visual->blue_mask    = 0x000000FF;
@@ -874,13 +876,10 @@ TkGetDefaultScreenName(
     Tcl_Interp *interp,		/* Not used. */
     const char *screenName)		/* If NULL, use default string. */
 {
-#if 0
     if ((screenName == NULL) || (screenName[0] == '\0')) {
 	screenName = macScreenName;
     }
     return screenName;
-#endif
-    return macScreenName;
 }
 
 /*
@@ -928,27 +927,23 @@ Tk_GetUserInactiveTime(
     timeObj = CFDictionaryGetValue(props, CFSTR("HIDIdleTime"));
 
     if (timeObj) {
-	CFTypeID type = CFGetTypeID(timeObj);
-
-	if (type == CFDataGetTypeID()) { /* Jaguar */
-	    CFDataGetBytes((CFDataRef) timeObj,
-		    CFRangeMake(0, sizeof(time)), (UInt8 *) &time);
-	    /* Convert nanoseconds to milliseconds. */
-	    /* ret /= kMillisecondScale; */
-	    ret = (long) (time/kMillisecondScale);
-	} else if (type == CFNumberGetTypeID()) { /* Panther+ */
 	    CFNumberGetValue((CFNumberRef)timeObj,
 		    kCFNumberSInt64Type, &time);
 	    /* Convert nanoseconds to milliseconds. */
-	    /* ret /= kMillisecondScale; */
 	    ret = (long) (time/kMillisecondScale);
-	} else {
-	    ret = -1l;
-	}
     }
     /* Cleanup */
     CFRelease(props);
 
+    /*
+     * If the idle time reported by the system is larger than the elapsed
+     * time since the last reset, return the elapsed time.
+     */
+    long elapsed = (long)(TkpGetMS() - lastInactivityReset);
+    if (ret > elapsed) {
+    	ret = elapsed;
+    }
+
     return ret;
 }
 
@@ -973,28 +968,7 @@ void
 Tk_ResetUserInactiveTime(
     Display *dpy)
 {
-    IOGPoint loc = {0, 0};
-    kern_return_t kr;
-    NXEvent nullEvent = {NX_NULLEVENT, {0, 0}, 0, -1, 0};
-    enum { kNULLEventPostThrottle = 10 };
-    static io_connect_t io_connection = MACH_PORT_NULL;
-
-    if (io_connection == MACH_PORT_NULL) {
-	io_service_t service = IOServiceGetMatchingService(
-		kIOMasterPortDefault, IOServiceMatching(kIOHIDSystemClass));
-
-	if (service == MACH_PORT_NULL) {
-	    return;
-	}
-	kr = IOServiceOpen(service, mach_task_self(), kIOHIDParamConnectType,
-		&io_connection);
-	IOObjectRelease(service);
-	if (kr != KERN_SUCCESS) {
-	    return;
-	}
-    }
-    kr = IOHIDPostEvent(io_connection, NX_NULLEVENT, loc, &nullEvent.data,
-	    FALSE, 0, FALSE);
+    lastInactivityReset = TkpGetMS();
 }
 
 /*
