@@ -13,7 +13,7 @@
 #include "tkInt.h"
 #include "tk3d.h"
 
-#if !(defined(__WIN32__) || defined(MAC_OSX_TK))
+#if !(defined(_WIN32) || defined(MAC_OSX_TK))
 #include "tkUnixInt.h"
 #endif
 
