@@ -11,7 +11,21 @@
  */
 
 #include "tkInt.h"
-#include <X11/XKBlib.h>
+
+/*
+** Bug [3607830]: Before using Xkb, it must be initialized.  TkpOpenDisplay
+**                does this and sets the USE_XKB flag if xkb is supported.
+**                (should this be function ptr?)
+*/
+#ifdef HAVE_XKBKEYCODETOKEYSYM
+#  include <X11/XKBlib.h>
+#else
+#  define XkbKeycodeToKeysym(D,K,G,L) XKeycodeToKeysym(D,K,L)
+#endif
+#define TkKeycodeToKeysym(D,K,G,L)		\
+    ((D)->flags & TK_DISPLAY_USE_XKB) ?		\
+      XkbKeycodeToKeysym((D)->display,K,G,L) :	\
+      XKeycodeToKeysym((D)->display,K,L)
 
 /*
  * Prototypes for local functions defined in this file:
@@ -111,6 +125,17 @@ TkpGetString(
 	return Tcl_DStringValue(dsPtr);
     }
 
+    /*
+     * Only do this for KeyPress events, otherwise
+     * further Xlib function behavior might be undefined.
+     */
+
+    if (eventPtr->type != KeyPress) {
+	len = 0;
+	Tcl_DStringSetLength(dsPtr, len);
+	goto done;
+    }
+
 #ifdef TK_USE_INPUT_METHODS
     if ((winPtr->dispPtr->flags & TK_DISPLAY_USE_IM)
 	    && (winPtr->inputContext != NULL)
@@ -203,6 +228,7 @@ TkpGetString(
      * from having to reenter the XIM engine. [Bug 1373712]
      */
 
+done:
     kePtr->charValuePtr = ckalloc(len + 1);
     kePtr->charValueLen = len;
     memcpy(kePtr->charValuePtr, Tcl_DStringValue(dsPtr), (unsigned) len + 1);
@@ -211,7 +237,7 @@ TkpGetString(
 
 /*
  * When mapping from a keysym to a keycode, need information about the
- * modifier state to be used so that when they call XkbKeycodeToKeysym taking
+ * modifier state to be used so that when they call TkKeycodeToKeysym taking
  * into account the xkey.state, they will get back the original keysym.
  */
 
@@ -221,30 +247,44 @@ TkpSetKeycodeAndState(
     KeySym keySym,
     XEvent *eventPtr)
 {
-    Display *display = Tk_Display(tkwin);
-    int state;
+    TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
+    int state, mincode, maxcode;
     KeyCode keycode;
 
     if (keySym == NoSymbol) {
 	keycode = 0;
     } else {
-	keycode = XKeysymToKeycode(display, keySym);
-	if (keycode != 0) {
-	    for (state = 0; state < 4; state++) {
-		if (XkbKeycodeToKeysym(display, keycode, 0, state) == keySym){
-		    if (state & 1) {
-			eventPtr->xkey.state |= ShiftMask;
-		    }
-		    if (state & 2) {
-			TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
-
-			eventPtr->xkey.state |= dispPtr->modeModMask;
-		    }
-		    break;
+	keycode = XKeysymToKeycode(dispPtr->display, keySym);
+    }
+    eventPtr->xkey.keycode = keycode;
+    if (keycode != 0) {
+	for (state = 0; state < 4; state++) {
+	    if (XLookupKeysym(&eventPtr->xkey, state) == keySym) {
+		if (state & 1) {
+		    eventPtr->xkey.state |= ShiftMask;
 		}
+		if (state & 2) {
+		    eventPtr->xkey.state |= dispPtr->modeModMask;
+		}
+		break;
 	    }
 	}
     }
+
+    /*
+     * Filter keycodes out of range, otherwise further Xlib function
+     * behavior might be undefined, in particular XIM could cause crashes.
+     */
+
+    mincode = 0;
+    maxcode = -1;
+    XDisplayKeycodes(dispPtr->display, &mincode, &maxcode);
+    if (keycode < mincode) {
+	keycode = mincode;
+    } else if (keycode > maxcode) {
+	keycode = maxcode;
+    }
+
     eventPtr->xkey.keycode = keycode;
 }
 
@@ -276,6 +316,15 @@ TkpGetKeySym(
     int index;
     TkKeyEvent* kePtr = (TkKeyEvent*) eventPtr;
 
+    /*
+     * Refresh the mapping information if it's stale. This must happen before
+     * we do any input method processing. [Bug 3599312]
+     */
+
+    if (dispPtr->bindInfoStale) {
+	TkpInitKeymapInfo(dispPtr);
+    }
+
 #ifdef TK_USE_INPUT_METHODS
     /*
      * If input methods are active, we may already have determined a keysym.
@@ -288,6 +337,7 @@ TkpGetKeySym(
 	    Tcl_DString ds;
 	    TkWindow *winPtr = (TkWindow *)
 		Tk_IdToWindow(eventPtr->xany.display, eventPtr->xany.window);
+
 	    Tcl_DStringInit(&ds);
 	    (void) TkpGetString(winPtr, eventPtr, &ds);
 	    Tcl_DStringFree(&ds);
@@ -298,14 +348,6 @@ TkpGetKeySym(
     }
 #endif
 
-    /*
-     * Refresh the mapping information if it's stale
-     */
-
-    if (dispPtr->bindInfoStale) {
-	TkpInitKeymapInfo(dispPtr);
-    }
-
     /*
      * Figure out which of the four slots in the keymap vector to use for this
      * key. Refer to Xlib documentation for more info on how this computation
@@ -321,7 +363,7 @@ TkpGetKeySym(
 	    && (eventPtr->xkey.state & LockMask))) {
 	index += 1;
     }
-    sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode, 0,
+    sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode, 0,
 	    index);
 
     /*
@@ -336,7 +378,7 @@ TkpGetKeySym(
 		|| ((sym >= XK_Agrave) && (sym <= XK_Odiaeresis))
 		|| ((sym >= XK_Ooblique) && (sym <= XK_Thorn)))) {
 	    index &= ~1;
-	    sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
+	    sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode,
 		    0, index);
 	}
     }
@@ -347,7 +389,7 @@ TkpGetKeySym(
      */
 
     if ((index & 1) && (sym == NoSymbol)) {
-	sym = XkbKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
+	sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode,
 		0, index & ~1);
     }
     return sym;
@@ -397,7 +439,7 @@ TkpInitKeymapInfo(
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = XkbKeycodeToKeysym(dispPtr->display, *codePtr, 0, 0);
+	keysym = TkKeycodeToKeysym(dispPtr, *codePtr, 0, 0);
 	if (keysym == XK_Shift_Lock) {
 	    dispPtr->lockUsage = LU_SHIFT;
 	    break;
@@ -423,7 +465,7 @@ TkpInitKeymapInfo(
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = XkbKeycodeToKeysym(dispPtr->display, *codePtr, 0, 0);
+	keysym = TkKeycodeToKeysym(dispPtr, *codePtr, 0, 0);
 	if (keysym == XK_Mode_switch) {
 	    dispPtr->modeModMask |= ShiftMask << (i/modMapPtr->max_keypermod);
 	}
