@@ -51,9 +51,9 @@ MODULE_SCOPE void	TkpDrawCheckIndicator(Tk_Window tkwin,
 static void		SetHelpMenu(TkMenu *menuPtr);
 static void		DrawMenuEntryAccelerator(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Drawable d, GC gc,
-			    Tk_Font tkfont, CONST Tk_FontMetrics *fmPtr,
-			    Tk_3DBorder activeBorder, int x, int y,
-			    int width, int height, int drawArrow);
+			    Tk_Font tkfont, const Tk_FontMetrics *fmPtr,
+			    Tk_3DBorder activeBorder, Tk_3DBorder bgBorder,
+			    int x, int y, int width, int height, int drawArrow);
 static void		DrawMenuEntryBackground(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Drawable d,
 			    Tk_3DBorder activeBorder, Tk_3DBorder bgBorder,
@@ -62,42 +62,42 @@ static void		DrawMenuEntryIndicator(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Drawable d,
 			    Tk_3DBorder border, XColor *indicatorColor,
 			    XColor *disableColor, Tk_Font tkfont,
-			    CONST Tk_FontMetrics *fmPtr, int x, int y,
+			    const Tk_FontMetrics *fmPtr, int x, int y,
 			    int width, int height);
 static void		DrawMenuEntryLabel(TkMenu * menuPtr,
 			    TkMenuEntry *mePtr, Drawable d, GC gc,
-			    Tk_Font tkfont, CONST Tk_FontMetrics *fmPtr,
+			    Tk_Font tkfont, const Tk_FontMetrics *fmPtr,
 			    int x, int y, int width, int height);
 static void		DrawMenuSeparator(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Drawable d, GC gc,
-			    Tk_Font tkfont, CONST Tk_FontMetrics *fmPtr,
+			    Tk_Font tkfont, const Tk_FontMetrics *fmPtr,
 			    int x, int y, int width, int height);
 static void		DrawTearoffEntry(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Drawable d, GC gc,
-			    Tk_Font tkfont, CONST Tk_FontMetrics *fmPtr,
+			    Tk_Font tkfont, const Tk_FontMetrics *fmPtr,
 			    int x, int y, int width, int height);
 static void		DrawMenuUnderline(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Drawable d, GC gc,
-			    Tk_Font tkfont, CONST Tk_FontMetrics *fmPtr,
+			    Tk_Font tkfont, const Tk_FontMetrics *fmPtr,
 			    int x, int y, int width, int height);
 static void		GetMenuAccelGeometry(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Tk_Font tkfont,
-			    CONST Tk_FontMetrics *fmPtr, int *widthPtr,
+			    const Tk_FontMetrics *fmPtr, int *widthPtr,
 			    int *heightPtr);
 static void		GetMenuLabelGeometry(TkMenuEntry *mePtr,
-			    Tk_Font tkfont, CONST Tk_FontMetrics *fmPtr,
+			    Tk_Font tkfont, const Tk_FontMetrics *fmPtr,
 			    int *widthPtr, int *heightPtr);
 static void		GetMenuIndicatorGeometry(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Tk_Font tkfont,
-			    CONST Tk_FontMetrics *fmPtr,
+			    const Tk_FontMetrics *fmPtr,
 			    int *widthPtr, int *heightPtr);
 static void		GetMenuSeparatorGeometry(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Tk_Font tkfont,
-			    CONST Tk_FontMetrics *fmPtr,
+			    const Tk_FontMetrics *fmPtr,
 			    int *widthPtr, int *heightPtr);
 static void		GetTearoffEntryGeometry(TkMenu *menuPtr,
 			    TkMenuEntry *mePtr, Tk_Font tkfont,
-			    CONST Tk_FontMetrics *fmPtr, int *widthPtr,
+			    const Tk_FontMetrics *fmPtr, int *widthPtr,
 			    int *heightPtr);
 
 /*
@@ -294,7 +294,7 @@ void
 TkpSetMainMenubar(
     Tcl_Interp *interp,
     Tk_Window tkwin,
-    char *menuName)
+    const char *menuName)
 {
     /*
      * Nothing to do.
@@ -324,10 +324,12 @@ GetMenuIndicatorGeometry(
     TkMenu *menuPtr,		/* The menu we are drawing. */
     TkMenuEntry *mePtr,		/* The entry we are interested in. */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalculated metrics */
     int *widthPtr,		/* The resulting width */
     int *heightPtr)		/* The resulting height */
 {
+    int borderWidth;
+
     if ((mePtr->type == CHECK_BUTTON_ENTRY)
 	    || (mePtr->type == RADIO_BUTTON_ENTRY)) {
 	if (!mePtr->hideMargin && mePtr->indicatorOn) {
@@ -352,23 +354,18 @@ GetMenuIndicatorGeometry(
 		}
 	    }
 	} else {
-	    int borderWidth;
-
-	    Tk_GetPixelsFromObj(NULL, menuPtr->tkwin,
-		    menuPtr->borderWidthPtr, &borderWidth);
+	    Tk_GetPixelsFromObj(NULL, menuPtr->tkwin, menuPtr->borderWidthPtr,
+		    &borderWidth);
 	    *heightPtr = 0;
 	    *widthPtr = borderWidth;
 	}
     } else {
-	int borderWidth;
-
 	Tk_GetPixelsFromObj(NULL, menuPtr->tkwin, menuPtr->borderWidthPtr,
 		&borderWidth);
-        *heightPtr = 0;
-        *widthPtr = borderWidth;
+	*heightPtr = 0;
+	*widthPtr = borderWidth;
     }
 }
-
 
 /*
  *----------------------------------------------------------------------
@@ -391,18 +388,17 @@ GetMenuAccelGeometry(
     TkMenu *menuPtr,		/* The menu was are drawing */
     TkMenuEntry *mePtr,		/* The entry we are getting the geometry for */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated font metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalculated font metrics */
     int *widthPtr,		/* The width of the acclerator area */
     int *heightPtr)		/* The height of the accelerator area */
 {
     *heightPtr = fmPtr->linespace;
     if (mePtr->type == CASCADE_ENTRY) {
     	*widthPtr = 2 * CASCADE_ARROW_WIDTH;
-    } else if ((menuPtr->menuType != MENUBAR)
-	    && (mePtr->accelPtr != NULL)) {
-	char *accel = Tcl_GetStringFromObj(mePtr->accelPtr, NULL);
+    } else if ((menuPtr->menuType != MENUBAR) && (mePtr->accelPtr != NULL)) {
+	const char *accel = Tcl_GetString(mePtr->accelPtr);
 
-    	*widthPtr = Tk_TextWidth(tkfont, accel, mePtr->accelLength);
+	*widthPtr = Tk_TextWidth(tkfont, accel, mePtr->accelLength);
     } else {
     	*widthPtr = 0;
     }
@@ -483,8 +479,9 @@ DrawMenuEntryAccelerator(
     Drawable d,			/* The drawable we are drawing into */
     GC gc,			/* The precalculated gc to draw with */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalculated metrics */
     Tk_3DBorder activeBorder,	/* The border for an active item */
+    Tk_3DBorder bgBorder,	/* The background border */
     int x,			/* Left coordinate of entry rect */
     int y,			/* Top coordinate of entry rect */
     int width,			/* Width of entry */
@@ -508,20 +505,21 @@ DrawMenuEntryAccelerator(
 	    &activeBorderWidth);
     if ((mePtr->type == CASCADE_ENTRY) && drawArrow) {
     	points[0].x = x + width - borderWidth - activeBorderWidth
-	        - CASCADE_ARROW_WIDTH;
+		- CASCADE_ARROW_WIDTH;
     	points[0].y = y + (height - CASCADE_ARROW_HEIGHT)/2;
     	points[1].x = points[0].x;
     	points[1].y = points[0].y + CASCADE_ARROW_HEIGHT;
     	points[2].x = points[0].x + CASCADE_ARROW_WIDTH;
     	points[2].y = points[0].y + CASCADE_ARROW_HEIGHT/2;
-    	Tk_Fill3DPolygon(menuPtr->tkwin, d, activeBorder, points, 3,
-		DECORATION_BORDER_WIDTH,
+    	Tk_Fill3DPolygon(menuPtr->tkwin, d,
+		(mePtr->state == ENTRY_ACTIVE) ? activeBorder : bgBorder,
+		points, 3, DECORATION_BORDER_WIDTH,
 	    	(menuPtr->postedCascade == mePtr)
 	    	? TK_RELIEF_SUNKEN : TK_RELIEF_RAISED);
     } else if (mePtr->accelPtr != NULL) {
-	char *accel = Tcl_GetStringFromObj(mePtr->accelPtr, NULL);
+	const char *accel = Tcl_GetString(mePtr->accelPtr);
 	int left = x + mePtr->labelWidth + activeBorderWidth
-	        + mePtr->indicatorSpace;
+		+ mePtr->indicatorSpace;
 
 	if (menuPtr->menuType == MENUBAR) {
 	    left += 5;
@@ -557,7 +555,7 @@ DrawMenuEntryIndicator(
     XColor *indicatorColor,	/* The color to draw indicators with */
     XColor *disableColor,	/* The color use use when disabled */
     Tk_Font tkfont,		/* The font to draw with */
-    CONST Tk_FontMetrics *fmPtr,/* The font metrics of the font */
+    const Tk_FontMetrics *fmPtr,/* The font metrics of the font */
     int x,			/* The left of the entry rect */
     int y,			/* The top of the entry rect */
     int width,			/* Width of menu entry */
@@ -629,7 +627,7 @@ DrawMenuSeparator(
     Drawable d,			/* The drawable we are using */
     GC gc,			/* The gc to draw into */
     Tk_Font tkfont,		/* The font to draw with */
-    CONST Tk_FontMetrics *fmPtr,/* The font metrics from the font */
+    const Tk_FontMetrics *fmPtr,/* The font metrics from the font */
     int x, int y,
     int width, int height)
 {
@@ -642,7 +640,7 @@ DrawMenuSeparator(
 
     points[0].x = x;
     points[0].y = y + height/2;
-    points[1].x = width - 1;
+    points[1].x = x + width - 1;
     points[1].y = points[0].y;
     border = Tk_Get3DBorderFromObj(menuPtr->tkwin, menuPtr->borderPtr);
     Tk_Draw3DPolygon(menuPtr->tkwin, d, border, points, 2, 1,
@@ -672,7 +670,7 @@ DrawMenuEntryLabel(
     Drawable d,			/* What we are drawing into. */
     GC gc,			/* The gc we are drawing into.*/
     Tk_Font tkfont,		/* The precalculated font. */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated font metrics. */
+    const Tk_FontMetrics *fmPtr,/* The precalculated font metrics. */
     int x,			/* Left edge. */
     int y,			/* Top edge. */
     int width,			/* width of entry. */
@@ -701,12 +699,14 @@ DrawMenuEntryLabel(
 	haveImage = 1;
     } else if (mePtr->bitmapPtr != NULL) {
 	Pixmap bitmap = Tk_GetBitmapFromObj(menuPtr->tkwin, mePtr->bitmapPtr);
+
 	Tk_SizeOfBitmap(menuPtr->display, bitmap, &imageWidth, &imageHeight);
 	haveImage = 1;
     }
     if (!haveImage || (mePtr->compound != COMPOUND_NONE)) {
 	if (mePtr->labelLength > 0) {
-	    char *label = Tcl_GetStringFromObj(mePtr->labelPtr, NULL);
+	    const char *label = Tcl_GetString(mePtr->labelPtr);
+
 	    textWidth = Tk_TextWidth(tkfont, label, mePtr->labelLength);
 	    textHeight = fmPtr->linespace;
 	    haveText = 1;
@@ -719,6 +719,7 @@ DrawMenuEntryLabel(
 
     if (haveImage && haveText) {
 	int fullWidth = (imageWidth > textWidth ? imageWidth : textWidth);
+
 	switch ((enum compound) mePtr->compound) {
 	case COMPOUND_TOP:
 	    textXOffset = (fullWidth - textWidth)/2;
@@ -801,7 +802,7 @@ DrawMenuEntryLabel(
 	int baseline = y + (height + fmPtr->ascent - fmPtr->descent) / 2;
 
     	if (mePtr->labelLength > 0) {
-	    char *label = Tcl_GetStringFromObj(mePtr->labelPtr, NULL);
+	    const char *label = Tcl_GetString(mePtr->labelPtr);
 
 	    Tk_DrawChars(menuPtr->display, d, gc, tkfont, label,
 		    mePtr->labelLength, leftEdge + textXOffset,
@@ -849,7 +850,7 @@ DrawMenuUnderline(
     Drawable d,			/* What we are drawing into */
     GC gc,			/* The gc to draw into */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated font metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalculated font metrics */
     int x, int y,
     int width, int height)
 {
@@ -863,9 +864,9 @@ DrawMenuUnderline(
 	Tcl_GetUnicodeFromObj(mePtr->labelPtr, &len);
 	if (mePtr->underline < len) {
 	    int activeBorderWidth, leftEdge;
-	    CONST char *label, *start, *end;
+	    const char *label, *start, *end;
 
-	    label = Tcl_GetStringFromObj(mePtr->labelPtr, NULL);
+	    label = Tcl_GetString(mePtr->labelPtr);
 	    start = Tcl_UtfAtIndex(label, mePtr->underline);
 	    end = Tcl_UtfNext(start);
 
@@ -929,7 +930,7 @@ GetMenuSeparatorGeometry(
     TkMenu *menuPtr,		/* The menu we are measuring */
     TkMenuEntry *mePtr,		/* The entry we are measuring */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalcualted font metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalcualted font metrics */
     int *widthPtr,		/* The resulting width */
     int *heightPtr)		/* The resulting height */
 {
@@ -958,7 +959,7 @@ GetTearoffEntryGeometry(
     TkMenu *menuPtr,		/* The menu we are drawing */
     TkMenuEntry *mePtr,		/* The entry we are measuring */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated font metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalculated font metrics */
     int *widthPtr,		/* The resulting width */
     int *heightPtr)		/* The resulting height */
 {
@@ -1074,7 +1075,6 @@ TkpComputeMenubarGeometry(
 	    if (mePtr->entryFlags & ENTRY_HELP_MENU) {
 		helpMenuIndex = i;
 	    } else if (x + mePtr->width + borderWidth > maxWindowWidth) {
-
 		if (i == lastRowBreak) {
 		    mePtr->y = y;
 		    mePtr->x = x;
@@ -1085,7 +1085,7 @@ TkpComputeMenubarGeometry(
 		    x = borderWidth;
 		    for (j = lastRowBreak; j < i; j++) {
 			menuPtr->entries[j]->y = y + currentRowHeight
-			        - menuPtr->entries[j]->height;
+				- menuPtr->entries[j]->height;
 			menuPtr->entries[j]->x = x;
 			x += menuPtr->entries[j]->width;
 		    }
@@ -1179,7 +1179,7 @@ DrawTearoffEntry(
     Drawable d,			/* The drawable we are drawing into */
     GC gc,			/* The gc we are drawing with */
     Tk_Font tkfont,		/* The font we are drawing with */
-    CONST Tk_FontMetrics *fmPtr,/* The metrics we are drawing with */
+    const Tk_FontMetrics *fmPtr,/* The metrics we are drawing with */
     int x, int y,
     int width, int height)
 {
@@ -1195,7 +1195,7 @@ DrawTearoffEntry(
     points[0].y = y + height/2;
     points[1].y = points[0].y;
     segmentWidth = 6;
-    maxX = width - 1;
+    maxX = x + width - 1;
     border = Tk_Get3DBorderFromObj(menuPtr->tkwin, menuPtr->borderPtr);
 
     while (points[0].x < maxX) {
@@ -1259,7 +1259,7 @@ TkpInitializeMenuBindings(
 
 static void
 SetHelpMenu(
-     TkMenu *menuPtr)		/* The menu we are checking */
+    TkMenu *menuPtr)		/* The menu we are checking */
 {
     TkMenuEntry *cascadeEntryPtr;
     int useMotifHelp = 0;
@@ -1317,25 +1317,24 @@ SetHelpMenu(
 
 void
 TkpDrawMenuEntry(
-    TkMenuEntry *mePtr,		    /* The entry to draw */
-    Drawable d,			    /* What to draw into */
-    Tk_Font tkfont,		    /* Precalculated font for menu */
-    CONST Tk_FontMetrics *menuMetricsPtr,
-				    /* Precalculated metrics for menu */
-    int x,			    /* X-coordinate of topleft of entry */
-    int y,			    /* Y-coordinate of topleft of entry */
-    int width,			    /* Width of the entry rectangle */
-    int height,			    /* Height of the current rectangle */
-    int strictMotif,		    /* Boolean flag */
-    int drawArrow)		    /* Whether or not to draw the cascade
-				     * arrow for cascade items. Only applies
-				     * to Windows. */
+    TkMenuEntry *mePtr,		/* The entry to draw */
+    Drawable d,			/* What to draw into */
+    Tk_Font tkfont,		/* Precalculated font for menu */
+    const Tk_FontMetrics *menuMetricsPtr,
+				/* Precalculated metrics for menu */
+    int x,			/* X-coordinate of topleft of entry */
+    int y,			/* Y-coordinate of topleft of entry */
+    int width,			/* Width of the entry rectangle */
+    int height,			/* Height of the current rectangle */
+    int strictMotif,		/* Boolean flag */
+    int drawArrow)		/* Whether or not to draw the cascade arrow
+				 * for cascade items. */
 {
     GC gc, indicatorGC;
     XColor *indicatorColor, *disableColor = NULL;
     TkMenu *menuPtr = mePtr->menuPtr;
     Tk_3DBorder bgBorder, activeBorder;
-    CONST Tk_FontMetrics *fmPtr;
+    const Tk_FontMetrics *fmPtr;
     Tk_FontMetrics entryMetrics;
     int padY = (menuPtr->menuType == MENUBAR) ? 3 : 0;
     int adjustedY = y + padY;
@@ -1358,7 +1357,7 @@ TkpDrawMenuEntry(
     		cascadeEntryPtr != NULL;
     		cascadeEntryPtr = cascadeEntryPtr->nextCascadePtr) {
 	    if (cascadeEntryPtr->namePtr != NULL) {
-		char *name = Tcl_GetString(cascadeEntryPtr->namePtr);
+		const char *name = Tcl_GetString(cascadeEntryPtr->namePtr);
 
 		if (strcmp(name, Tk_PathName(menuPtr->tkwin)) == 0) {
 		    if (cascadeEntryPtr->state == ENTRY_DISABLED) {
@@ -1437,7 +1436,8 @@ TkpDrawMenuEntry(
 	DrawMenuEntryLabel(menuPtr, mePtr, d, gc, tkfont, fmPtr, x, adjustedY,
 		width, adjustedHeight);
 	DrawMenuEntryAccelerator(menuPtr, mePtr, d, gc, tkfont, fmPtr,
-		activeBorder, x, adjustedY, width, adjustedHeight, drawArrow);
+		activeBorder, bgBorder, x, adjustedY, width, adjustedHeight,
+		drawArrow);
 	if (!mePtr->hideMargin) {
 	    if (mePtr->state == ENTRY_ACTIVE) {
 		bgBorder = activeBorder;
@@ -1470,7 +1470,7 @@ static void
 GetMenuLabelGeometry(
     TkMenuEntry *mePtr,		/* The entry we are computing */
     Tk_Font tkfont,		/* The precalculated font */
-    CONST Tk_FontMetrics *fmPtr,/* The precalculated metrics */
+    const Tk_FontMetrics *fmPtr,/* The precalculated metrics */
     int *widthPtr,		/* The resulting width of the label portion */
     int *heightPtr)		/* The resulting height of the label
 				 * portion */
@@ -1483,6 +1483,7 @@ GetMenuLabelGeometry(
 	haveImage = 1;
     } else if (mePtr->bitmapPtr != NULL) {
 	Pixmap bitmap = Tk_GetBitmapFromObj(menuPtr->tkwin, mePtr->bitmapPtr);
+
     	Tk_SizeOfBitmap(menuPtr->display, bitmap, widthPtr, heightPtr);
 	haveImage = 1;
     } else {
@@ -1501,9 +1502,9 @@ GetMenuLabelGeometry(
 
     	if (mePtr->labelPtr != NULL) {
 	    int textWidth;
-	    char *label = Tcl_GetStringFromObj(mePtr->labelPtr, NULL);
-	    textWidth = Tk_TextWidth(tkfont, label, mePtr->labelLength);
+	    const char *label = Tcl_GetString(mePtr->labelPtr);
 
+	    textWidth = Tk_TextWidth(tkfont, label, mePtr->labelLength);
 	    if ((mePtr->compound != COMPOUND_NONE) && haveImage) {
 		switch ((enum compound) mePtr->compound) {
 		case COMPOUND_TOP:
@@ -1686,8 +1687,8 @@ TkpComputeStandardMenuGeometry(
 	    	accelWidth = width;
 	    }
 
-	    GetMenuIndicatorGeometry(menuPtr, mePtr, tkfont,
-	    	    fmPtr, &width, &height);
+	    GetMenuIndicatorGeometry(menuPtr, mePtr, tkfont, fmPtr,
+		    &width, &height);
 	    if (height > mePtr->height) {
 	    	mePtr->height = height;
 	    }
@@ -1700,7 +1701,7 @@ TkpComputeStandardMenuGeometry(
 
 	    mePtr->height += 2 * activeBorderWidth + MENU_DIVIDER_HEIGHT;
     	}
-        mePtr->y = y;
+	mePtr->y = y;
 	y += mePtr->height;
 	if (y > windowHeight) {
 	    windowHeight = y;
@@ -1719,7 +1720,7 @@ TkpComputeStandardMenuGeometry(
 	menuPtr->entries[j]->entryFlags |= ENTRY_LAST_COLUMN;
     }
     windowWidth = x + indicatorSpace + labelWidth + accelWidth
-	    + 2 * activeBorderWidth + 2 * borderWidth;
+	    + 2 * activeBorderWidth + borderWidth;
 
     windowHeight += borderWidth;
 
@@ -1759,7 +1760,7 @@ TkpComputeStandardMenuGeometry(
 void
 TkpMenuNotifyToplevelCreate(
     Tcl_Interp *interp,		/* The interp the menu lives in. */
-    char *menuName)		/* The name of the menu to reconfigure. */
+    const char *menuName)	/* The name of the menu to reconfigure. */
 {
     /*
      * Nothing to do.
