@@ -261,11 +261,14 @@ RegOpen(
     unsigned long bytesAfter;
     Atom actualType;
     char **propertyPtr;
+    Tk_ErrorHandler handler;
 
     if (dispPtr->commTkwin == NULL) {
 	SendInit(interp, dispPtr);
     }
 
+    handler = Tk_CreateErrorHandler(dispPtr->display, -1, -1, -1, NULL, NULL);
+
     regPtr = ckalloc(sizeof(NameRegistry));
     regPtr->dispPtr = dispPtr;
     regPtr->locked = 0;
@@ -306,8 +309,11 @@ RegOpen(
 	XDeleteProperty(dispPtr->display,
 		RootWindow(dispPtr->display, 0),
 		dispPtr->registryProperty);
+        XSync(dispPtr->display, False);
     }
 
+    Tk_DeleteErrorHandler(handler);
+
     /*
      * Xlib placed an extra null byte after the end of the property, just to
      * make sure that it is always NULL-terminated. Be sure to include this
@@ -514,6 +520,11 @@ RegClose(
     NameRegistry *regPtr)	/* Pointer to a registry opened with a
 				 * previous call to RegOpen. */
 {
+    Tk_ErrorHandler handler;
+
+    handler = Tk_CreateErrorHandler(regPtr->dispPtr->display, -1, -1, -1,
+            NULL, NULL);
+
     if (regPtr->modified) {
 	if (!regPtr->locked && !localData.sendDebug) {
 	    Tcl_Panic("The name registry was modified without being locked!");
@@ -540,6 +551,8 @@ RegClose(
 
     XFlush(regPtr->dispPtr->display);
 
+    Tk_DeleteErrorHandler(handler);
+
     if (regPtr->property != NULL) {
 	if (regPtr->allocedByX) {
 	    XFree(regPtr->property);
@@ -823,7 +836,7 @@ Tk_SetAppName(
 	    riPtr->nextPtr = tsdPtr->interpListPtr;
 	    tsdPtr->interpListPtr = riPtr;
 	    riPtr->name = NULL;
-	    Tcl_CreateCommand(interp, "send", Tk_SendCmd, riPtr, DeleteProc);
+	    Tcl_CreateObjCommand(interp, "send", Tk_SendObjCmd, riPtr, DeleteProc);
 	    if (Tcl_IsSafe(interp)) {
 		Tcl_HideCommand(interp, "send", "send");
 	    }
@@ -914,7 +927,7 @@ Tk_SetAppName(
 /*
  *--------------------------------------------------------------
  *
- * Tk_SendCmd --
+ * Tk_SendObjCmd --
  *
  *	This function is invoked to process the "send" Tcl command. See the
  *	user documentation for details on what it does.
@@ -929,20 +942,25 @@ Tk_SetAppName(
  */
 
 int
-Tk_SendCmd(
+Tk_SendObjCmd(
     ClientData clientData,	/* Information about sender (only dispPtr
 				 * field is used). */
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const objv[])	/* Argument strings. */
 {
+    enum {
+	SEND_ASYNC, SEND_DISPLAYOF, SEND_LAST
+    };
+    static const char *const sendOptions[] = {
+	"-async",   "-displayof",   "--",  NULL
+    };
     TkWindow *winPtr;
     Window commWindow;
     PendingCommand pending;
     register RegisteredInterp *riPtr;
     const char *destName;
-    int result, c, async, i, firstArg;
-    size_t length;
+    int result, index, async, i, firstArg;
     Tk_RestrictProc *prevProc;
     ClientData prevArg;
     TkDisplay *dispPtr;
@@ -963,43 +981,31 @@ Tk_SendCmd(
     if (winPtr == NULL) {
 	return TCL_ERROR;
     }
-    for (i = 1; i < (argc-1); ) {
-	if (argv[i][0] != '-') {
+    for (i = 1; i < objc; i++) {
+	if (Tcl_GetIndexFromObjStruct(interp, objv[i], sendOptions,
+		sizeof(char *), "option", 0, &index) != TCL_OK) {
 	    break;
 	}
-	c = argv[i][1];
-	length = strlen(argv[i]);
-	if ((c == 'a') && (strncmp(argv[i], "-async", length) == 0)) {
-	    async = 1;
-	    i++;
-	} else if ((c == 'd') && (strncmp(argv[i], "-displayof",
-		length) == 0)) {
-	    winPtr = (TkWindow *) Tk_NameToWindow(interp, argv[i+1],
+	if (index == SEND_ASYNC) {
+	    ++async;
+	} else if (index == SEND_DISPLAYOF) {
+	    winPtr = (TkWindow *) Tk_NameToWindow(interp, Tcl_GetString(objv[++i]),
 		    (Tk_Window) winPtr);
 	    if (winPtr == NULL) {
 		return TCL_ERROR;
 	    }
-	    i += 2;
-	} else if (strcmp(argv[i], "--") == 0) {
+	} else if (index == SEND_LAST) {
 	    i++;
 	    break;
-	} else {
-	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
-		    "bad option \"%s\": must be -async, -displayof, or --",
-		    argv[i]));
-	    Tcl_SetErrorCode(interp, "TK", "SEND", "OPTION", NULL);
-	    return TCL_ERROR;
 	}
     }
 
-    if (argc < (i+2)) {
-	Tcl_SetObjResult(interp, Tcl_ObjPrintf("wrong # args: should be "
-		"\"%s ?-option value ...? interpName arg ?arg ...?\"",
-		argv[0]));
-	Tcl_SetErrorCode(interp, "TCL", "WRONGARGS", NULL);
+    if (objc < (i+2)) {
+	Tcl_WrongNumArgs(interp, 1, objv,
+		"?-option value ...? interpName arg ?arg ...?");
 	return TCL_ERROR;
     }
-    destName = argv[i];
+    destName = Tcl_GetString(objv[i]);
     firstArg = i+1;
 
     dispPtr = winPtr->dispPtr;
@@ -1023,14 +1029,14 @@ Tk_SendCmd(
 	Tcl_Preserve(riPtr);
 	localInterp = riPtr->interp;
 	Tcl_Preserve(localInterp);
-	if (firstArg == (argc-1)) {
-	    result = Tcl_EvalEx(localInterp, argv[firstArg], -1, TCL_EVAL_GLOBAL);
+	if (firstArg == (objc-1)) {
+	    result = Tcl_EvalEx(localInterp, Tcl_GetString(objv[firstArg]), -1, TCL_EVAL_GLOBAL);
 	} else {
 	    Tcl_DStringInit(&request);
-	    Tcl_DStringAppend(&request, argv[firstArg], -1);
-	    for (i = firstArg+1; i < argc; i++) {
+	    Tcl_DStringAppend(&request, Tcl_GetString(objv[firstArg]), -1);
+	    for (i = firstArg+1; i < objc; i++) {
 		Tcl_DStringAppend(&request, " ", 1);
-		Tcl_DStringAppend(&request, argv[i], -1);
+		Tcl_DStringAppend(&request, Tcl_GetString(objv[i]), -1);
 	    }
 	    result = Tcl_EvalEx(localInterp, Tcl_DStringValue(&request), -1, TCL_EVAL_GLOBAL);
 	    Tcl_DStringFree(&request);
@@ -1097,10 +1103,35 @@ Tk_SendCmd(
 	Tcl_DStringAppend(&request, buffer, -1);
     }
     Tcl_DStringAppend(&request, "\0-s ", 4);
-    Tcl_DStringAppend(&request, argv[firstArg], -1);
-    for (i = firstArg+1; i < argc; i++) {
+    Tcl_DStringAppend(&request, Tcl_GetString(objv[firstArg]), -1);
+    for (i = firstArg+1; i < objc; i++) {
 	Tcl_DStringAppend(&request, " ", 1);
-	Tcl_DStringAppend(&request, argv[i], -1);
+	Tcl_DStringAppend(&request, Tcl_GetString(objv[i]), -1);
+    }
+
+    if (!async) {
+	/*
+	 * Register the fact that we're waiting for a command to complete
+	 * (this is needed by SendEventProc and by AppendErrorProc to pass
+	 * back the command's results). Set up a timeout handler so that
+	 * we can check during long sends to make sure that the destination
+	 * application is still alive.
+	 *
+	 * We prepare the pending struct here in order to catch potential
+	 * early X errors from AppendPropCarefully() due to XSync().
+	 */
+
+	pending.serial = localData.sendSerial;
+	pending.dispPtr = dispPtr;
+	pending.target = destName;
+	pending.commWindow = commWindow;
+	pending.interp = interp;
+	pending.result = NULL;
+	pending.errorInfo = NULL;
+	pending.errorCode = NULL;
+	pending.gotResponse = 0;
+	pending.nextPtr = tsdPtr->pendingCommands;
+	tsdPtr->pendingCommands = &pending;
     }
     (void) AppendPropCarefully(dispPtr->display, commWindow,
 	    dispPtr->commProperty, Tcl_DStringValue(&request),
@@ -1115,26 +1146,6 @@ Tk_SendCmd(
 	return TCL_OK;
     }
 
-    /*
-     * Register the fact that we're waiting for a command to complete (this is
-     * needed by SendEventProc and by AppendErrorProc to pass back the
-     * command's results). Set up a timeout handler so that we can check
-     * during long sends to make sure that the destination application is
-     * still alive.
-     */
-
-    pending.serial = localData.sendSerial;
-    pending.dispPtr = dispPtr;
-    pending.target = destName;
-    pending.commWindow = commWindow;
-    pending.interp = interp;
-    pending.result = NULL;
-    pending.errorInfo = NULL;
-    pending.errorCode = NULL;
-    pending.gotResponse = 0;
-    pending.nextPtr = tsdPtr->pendingCommands;
-    tsdPtr->pendingCommands = &pending;
-
     /*
      * Enter a loop processing X events until the result comes in or the
      * target is declared to be dead. While waiting for a result, look only at
@@ -1948,44 +1959,59 @@ int
 TkpTestsendCmd(
     ClientData clientData,	/* Main window for application. */
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const objv[])		/* Argument strings. */
 {
+    enum {
+	TESTSEND_BOGUS, TESTSEND_PROP, TESTSEND_SERIAL
+    };
+    static const char *const testsendOptions[] = {
+	"bogus",   "prop",   "serial",  NULL
+    };
     TkWindow *winPtr = clientData;
+    Tk_ErrorHandler handler;
+    int index;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args; must be \"", argv[0],
-		" option ?arg ...?\"", NULL);
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv,
+		"option ?arg ...?");
 	return TCL_ERROR;
     }
 
-    if (strcmp(argv[1], "bogus") == 0) {
+    if (Tcl_GetIndexFromObjStruct(interp, objv[1], testsendOptions,
+		sizeof(char *), "option", 0, &index) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    if (index == TESTSEND_BOGUS) {
+        handler = Tk_CreateErrorHandler(winPtr->dispPtr->display, -1, -1, -1,
+                NULL, NULL);
 	XChangeProperty(winPtr->dispPtr->display,
 		RootWindow(winPtr->dispPtr->display, 0),
 		winPtr->dispPtr->registryProperty, XA_INTEGER, 32,
 		PropModeReplace,
 		(unsigned char *) "This is bogus information", 6);
-    } else if (strcmp(argv[1], "prop") == 0) {
+        Tk_DeleteErrorHandler(handler);
+    } else if (index == TESTSEND_PROP) {
 	int result, actualFormat;
 	unsigned long length, bytesAfter;
 	Atom actualType, propName;
 	char *property, **propertyPtr = &property, *p, *end;
 	Window w;
 
-	if ((argc != 4) && (argc != 5)) {
-	    Tcl_AppendResult(interp, "wrong # args; must be \"", argv[0],
-		    " prop window name ?value ?\"", NULL);
+	if ((objc != 4) && (objc != 5)) {
+		Tcl_WrongNumArgs(interp, 1, objv,
+			"prop window name ?value ?");
 	    return TCL_ERROR;
 	}
-	if (strcmp(argv[2], "root") == 0) {
+	if (strcmp(Tcl_GetString(objv[2]), "root") == 0) {
 	    w = RootWindow(winPtr->dispPtr->display, 0);
-	} else if (strcmp(argv[2], "comm") == 0) {
+	} else if (strcmp(Tcl_GetString(objv[2]), "comm") == 0) {
 	    w = Tk_WindowId(winPtr->dispPtr->commTkwin);
 	} else {
-	    w = strtoul(argv[2], &end, 0);
+	    w = strtoul(Tcl_GetString(objv[2]), &end, 0);
 	}
-	propName = Tk_InternAtom((Tk_Window) winPtr, argv[3]);
-	if (argc == 4) {
+	propName = Tk_InternAtom((Tk_Window) winPtr, Tcl_GetString(objv[3]));
+	if (objc == 4) {
 	    property = NULL;
 	    result = XGetWindowProperty(winPtr->dispPtr->display, w, propName,
 		    0, 100000, False, XA_STRING, &actualType, &actualFormat,
@@ -2002,30 +2028,31 @@ TkpTestsendCmd(
 	    if (property != NULL) {
 		XFree(property);
 	    }
-	} else if (argv[4][0] == 0) {
+	} else if (Tcl_GetString(objv[4])[0] == 0) {
+            handler = Tk_CreateErrorHandler(winPtr->dispPtr->display,
+                    -1, -1, -1, NULL, NULL);
 	    XDeleteProperty(winPtr->dispPtr->display, w, propName);
+            Tk_DeleteErrorHandler(handler);
 	} else {
 	    Tcl_DString tmp;
 
 	    Tcl_DStringInit(&tmp);
-	    for (p = Tcl_DStringAppend(&tmp, argv[4],
-		    (int) strlen(argv[4])); *p != 0; p++) {
+	    for (p = Tcl_DStringAppend(&tmp, Tcl_GetString(objv[4]),
+		    (int) strlen(Tcl_GetString(objv[4]))); *p != 0; p++) {
 		if (*p == '\n') {
 		    *p = 0;
 		}
 	    }
-
+            handler = Tk_CreateErrorHandler(winPtr->dispPtr->display,
+                    -1, -1, -1, NULL, NULL);
 	    XChangeProperty(winPtr->dispPtr->display, w, propName, XA_STRING,
 		    8, PropModeReplace, (unsigned char*)Tcl_DStringValue(&tmp),
 		    p-Tcl_DStringValue(&tmp));
+            Tk_DeleteErrorHandler(handler);
 	    Tcl_DStringFree(&tmp);
 	}
-    } else if (strcmp(argv[1], "serial") == 0) {
+    } else if (index == TESTSEND_SERIAL) {
 	Tcl_SetObjResult(interp, Tcl_NewIntObj(localData.sendSerial+1));
-    } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1],
-		"\": must be bogus, prop, or serial", NULL);
-	return TCL_ERROR;
     }
     return TCL_OK;
 }
